//
// SWIG typemaps for blocxx::String
// based on std::string from
// Roy M. LeCates
// October 23, 2002
//
// Michael Calmer
// March, 15, 2005
//
// Perl implementation


// ------------------------------------------------------------------------
// blocxx::string is typemapped by value
// This can prevent exporting methods which return a string
// in order for the user to modify it.
// However, I think I'll wait until someone asks for it...
// ------------------------------------------------------------------------

%include exception.i

%{
#include <blocxx/BLOCXX_config.h>
#include <blocxx/String.hpp>
%}


class blocxx::String;

/* Overloading check */
%typemap(typecheck) blocxx::String = char *;

// %typemap(typecheck) const blocxx::String & = char *;

%typecheck(SWIG_TYPECHECK_STRING) const blocxx::String & , const blocxx::String * {
  if(SvROK($input)) {
     SV *sv = (SV *)SvRV($input);
     if(!SvPOK(sv)) {
        $1 = 0;
     } else {
        $1 = 1;
     }
  } else {
     $1 = SvPOK($input) ? 1 : 0;
  }
}

%typecheck(SWIG_TYPECHECK_STRING) blocxx::String & , blocxx::String * {
  if(!SvROK($input)) {
     $1 = 0;
  } else {
     SV *sv = (SV *)SvRV($input);
     if(!SvPOK(sv)) {
        $1 = 0;
     } else {
        $1 = 1;
     }
  }
}


%typemap(in) blocxx::String {
    STRLEN len;
    const char *ptr = SvPV($input, len);
    if (!ptr) {
        SWIG_croak("Undefined variable in argument $argnum of $symname.");
    } else {
        $1 = blocxx::String(ptr, len);
    }
}

%typemap(in) 
             blocxx::String *(blocxx::String temp), 
             blocxx::String &(blocxx::String temp) { 
    if (!SvROK($input))
        SWIG_croak("Type error in argument $argnum of $symname. Expected a REFERENCE to a string.\n");
    SV *sv = (SV *)SvRV($input);
    if (!SvPOK(sv))
        SWIG_croak("Type error in argument $argnum of $symname. Expected a reference to a STRING.\n");
    STRLEN len;
    const char *ptr = SvPV(sv, len);
    if (!ptr) {
        SWIG_croak("Undefined variable in argument $argnum of $symname.");
    } else {
        temp = blocxx::String(ptr, len);
        $1 = &temp;
    }
}

%typemap(in)
             const blocxx::String *(blocxx::String temp),
             const blocxx::String &(blocxx::String temp) {
  const char *ptr = 0;
  STRLEN len;
  if(SvROK($input)) {
     SV *sv = (SV *)SvRV($input);
     if(!SvPOK(sv)) {
        SWIG_croak("Type error in argument $argnum of $symname. Expected a reference to a STRING.\n");
     }
     ptr = SvPV(sv, len);
  } else {
     ptr = SvPV($input, len) ;
  }
  if (!ptr) {
     SWIG_croak("Undefined variable in argument $argnum of $symname.");
  } else {
     temp = blocxx::String(ptr, len);
     $1 = &temp;
 }
}


%typemap(argout) blocxx::String*, blocxx::String&
{
    if(SvROK($input)) {
        SV *sv = (SV *)SvRV($input);
        sv_setpv(sv, $1->c_str());
    }
}

%typemap(out) blocxx::String {
    if (argvi >= items) EXTEND(sp, 1);	// bump stack ptr, if needed
    char *data = const_cast<char*>($1.c_str());
    sv_setpvn($result = sv_newmortal(), data, $1.length());
    ++argvi;
}

%typemap(out) const blocxx::String &, blocxx::String & {
    if (argvi >= items) EXTEND(sp, 1);	// bump stack ptr, if needed
    char *data = const_cast<char*>($1->c_str());
    sv_setpvn($result = sv_newmortal(), data, $1->length());
    ++argvi;
}

/*
%typemap(throws) const blocxx::String & {
  SWIG_croak($1.c_str());
}

%typemap(throws) blocxx::String {
  SWIG_croak($1.c_str());
}
*/


